--------------------------------------------------------------------------------------------
-- server.lua - Demonstrates a very simple server
--------------------------------------------------------------------------------------------
require ("configure_me");  --Include the file 'configure_me.lua'
require ("library");       --Include the file 'library.lua'

--Server main loop
while (1) do
   Sleep (200); --Don't use too much computing power
   MessagePump();

   --Check for 100 different tickets:
   for c=0,99 do
      local ticketfullfilename = ticket_path.."ticket"..string.format("%02i", c)..".txt";

      --Do we have a ticket file for that number?
      if (DoesFileExist (ticketfullfilename)) then
         INPUT = io.open(ticketfullfilename, "rb")
         verb = chomp(INPUT:read("*line"))

         --Mode: Analyze a file
         if (verb=="analyze") then
            inputfile = chomp(INPUT:read("*line"))

            OUTPUT = io.open (ticketfullfilename..".output", "w")

            --Check the input file:
            if (DoesFileExist(input_path..inputfile)) then

               --Create a new project and import the imput file:
               NewProject();
               
               if (projectImport(input_path..inputfile)) then
                  --Look for 100% identical projects, return 1 (the best) and give me the filename:
                  similarpojects = projectFindSimilarProjects (100, 1, ePrjFilename);

                  if (#similarpojects>0) then
                     --Get the list of versions from the project
                     versionlist = GetProjectVersions (similarpojects[2]);
                     size = #versionlist

                     OUTPUT:write ("versionlist\r\n");
                     OUTPUT:write (similarpojects[2]);
                     OUTPUT:write ("\r\n");
                     OUTPUT:write (size/2); --list contains name and description => size/2 is the real size
                     OUTPUT:write ("\r\n");

                     for i=1, size/2 do
                        OUTPUT:write (versionlist [i*2-1]);
                        OUTPUT:write ("\r\n");
                     end 
                  else
                     OUTPUT:write ("not found"); --No identical project found
                  end
               else
                  OUTPUT:write ("error\r\n"); --Internal error. Shouldn't happen unless you're not allowed to import that file (NOREAD)
                  OUTPUT:write ("Import failed: "..inputfile);
               end

               projectClose(TRUE);

               --Remove the uploaded:
               os.remove (input_path..inputfile);
            else
               OUTPUT:write ("error\r\n"); --Internal error. Did you forget to create the binary file or did you create the ticket before creating the binary file?
               OUTPUT:write ("file not found: "..inputfile);
            end

            OUTPUT:close();
         end

         --Mode: Return a file version
         if (verb=="getversion") then
            olsfile        = chomp(INPUT:read("*line"));
            versionsnumber = chomp(INPUT:read("*line"));  --0=Original, 1=first version, ...
            outputfilename = chomp(INPUT:read("*line"));

            OUTPUT = io.open (ticketfullfilename..".output", "w")

            if (OpenProjectVersion (olsfile, 1*versionsnumber)) then -- "1*" => cast to integer
               if (projectExport (ticket_path..outputfilename, eFiletypeBinary)) then
                  OUTPUT:write ("done"); --File exported successfully
               else
                  OUTPUT:write ("error\r\n"); --Internal error. No right to export or bad filename
                  OUTPUT:write ("export failed: "..olsfile);
               end

               projectClose ();
            else
               OUTPUT:write ("error\r\n"); --Internal error. The requested version does not exist.
               OUTPUT:write ("file not found: "..olsfile);
            end

            OUTPUT:close();
         end

         INPUT:close();

         --Remove the ticket file:
         os.remove (ticketfullfilename);
      end
   end

   --Check for quit ticket:
   if (DoesFileExist (ticket_path.."quit_ticket.txt")) then
      os.remove (ticket_path.."quit_ticket.txt");
      Quit(); --End the server
   end
end